@@ -2,16 +2,16 @@
 psycopg connection objects
 """
 
-# Copyright (C) 2020-2021 The Psycopg Team
+# Copyright (C) 2020 The Psycopg Team
 
 import logging
-import warnings
 import threading
 from types import TracebackType
 from typing import Any, Callable, cast, Dict, Generator, Generic, Iterator
 from typing import List, NamedTuple, Optional, Type, TypeVar, Union
 from typing import overload, TYPE_CHECKING
 from weakref import ref, ReferenceType
+from warnings import warn
 from functools import partial
 from contextlib import contextmanager
 
@@ -144,9 +144,9 @@ class BaseConnection(Generic[Row]):
         if hasattr(self, "_pool"):
             return
 
-        warnings.warn(
+        warn(
             f"connection {self} was deleted while still open."
-            f" Please use 'with' or '.close()' to close the connection",
+            " Please use 'with' or '.close()' to close the connection",
             ResourceWarning,
         )
 
@@ -200,9 +200,7 @@ class BaseConnection(Generic[Row]):
         # Base implementation, not thread safe.
         # Subclasses must call it holding a lock
         self._check_intrans("isolation_level")
-        self._isolation_level = (
-            IsolationLevel(value) if value is not None else None
-        )
+        self._isolation_level = IsolationLevel(value) if value is not None else None
         self._begin_statement = b""
 
     @property
@@ -288,12 +286,18 @@ class BaseConnection(Generic[Row]):
     def add_notice_handler(self, callback: NoticeHandler) -> None:
         """
         Register a callable to be invoked when a notice message is received.
+
+        :param callback: the callback to call upon message received.
+        :type callback: Callable[[~psycopg.errors.Diagnostic], None]
         """
         self._notice_handlers.append(callback)
 
     def remove_notice_handler(self, callback: NoticeHandler) -> None:
         """
         Unregister a notice message callable previously registered.
+
+        :param callback: the callback to remove.
+        :type callback: Callable[[~psycopg.errors.Diagnostic], None]
         """
         self._notice_handlers.remove(callback)
 
@@ -310,19 +314,23 @@ class BaseConnection(Generic[Row]):
             try:
                 cb(diag)
             except Exception as ex:
-                logger.exception(
-                    "error processing notice callback '%s': %s", cb, ex
-                )
+                logger.exception("error processing notice callback '%s': %s", cb, ex)
 
     def add_notify_handler(self, callback: NotifyHandler) -> None:
         """
         Register a callable to be invoked whenever a notification is received.
+
+        :param callback: the callback to call upon notification received.
+        :type callback: Callable[[~psycopg.Notify], None]
         """
         self._notify_handlers.append(callback)
 
     def remove_notify_handler(self, callback: NotifyHandler) -> None:
         """
         Unregister a notification callable previously registered.
+
+        :param callback: the callback to remove.
+        :type callback: Callable[[~psycopg.Notify], None]
         """
         self._notify_handlers.remove(callback)
 
@@ -344,7 +352,7 @@ class BaseConnection(Generic[Row]):
         """
         Number of times a query is executed before it is prepared.
 
-        - If it is set to 0, every query is prepared the first time is
+        - If it is set to 0, every query is prepared the first time it is
           executed.
         - If it is set to `!None`, prepared statements are disabled on the
           connection.
@@ -412,16 +420,12 @@ class BaseConnection(Generic[Row]):
         if result_format == Format.TEXT:
             self.pgconn.send_query(command)
         else:
-            self.pgconn.send_query_params(
-                command, None, result_format=result_format
-            )
+            self.pgconn.send_query_params(command, None, result_format=result_format)
 
         result = (yield from execute(self.pgconn))[-1]
         if result.status not in (ExecStatus.COMMAND_OK, ExecStatus.TUPLES_OK):
             if result.status == ExecStatus.FATAL_ERROR:
-                raise e.error_from_result(
-                    result, encoding=pgconn_encoding(self.pgconn)
-                )
+                raise e.error_from_result(result, encoding=pgconn_encoding(self.pgconn))
             else:
                 raise e.InterfaceError(
                     f"unexpected result {ExecStatus(result.status).name}"
@@ -436,7 +440,7 @@ class BaseConnection(Generic[Row]):
         if self.pgconn.status == ConnStatus.BAD:
             raise e.OperationalError("the connection is closed")
         raise e.InterfaceError(
-            f"cannot execute operations: the connection is"
+            "cannot execute operations: the connection is"
             f" in status {self.pgconn.status}"
         )
 
@@ -465,9 +469,7 @@ class BaseConnection(Generic[Row]):
             parts.append(b"READ ONLY" if self.read_only else b"READ WRITE")
 
         if self.deferrable is not None:
-            parts.append(
-                b"DEFERRABLE" if self.deferrable else b"NOT DEFERRABLE"
-            )
+            parts.append(b"DEFERRABLE" if self.deferrable else b"NOT DEFERRABLE")
 
         self._begin_statement = b" ".join(parts)
         return self._begin_statement
@@ -513,9 +515,7 @@ class Connection(BaseConnection[Row]):
     server_cursor_factory: Type[ServerCursor[Row]]
     row_factory: RowFactory[Row]
 
-    def __init__(
-        self, pgconn: "PGconn", row_factory: Optional[RowFactory[Row]] = None
-    ):
+    def __init__(self, pgconn: "PGconn", row_factory: Optional[RowFactory[Row]] = None):
         super().__init__(pgconn)
         self.row_factory = row_factory or cast(RowFactory[Row], tuple_row)
         self.lock = threading.Lock()
@@ -608,9 +608,7 @@ class Connection(BaseConnection[Row]):
             self.close()
 
     @classmethod
-    def _get_connection_params(
-        cls, conninfo: str, **kwargs: Any
-    ) -> Dict[str, Any]:
+    def _get_connection_params(cls, conninfo: str, **kwargs: Any) -> Dict[str, Any]:
         """Manipulate connection parameters before connecting.
 
         :param conninfo: Connection string as received by `~Connection.connect()`.
@@ -758,9 +756,7 @@ class Connection(BaseConnection[Row]):
                 ns = self.wait(notifies(self.pgconn))
             enc = pgconn_encoding(self.pgconn)
             for pgn in ns:
-                n = Notify(
-                    pgn.relname.decode(enc), pgn.extra.decode(enc), pgn.be_pid
-                )
+                n = Notify(pgn.relname.decode(enc), pgn.extra.decode(enc), pgn.be_pid)
                 yield n
 
     def wait(self, gen: PQGen[RV], timeout: Optional[float] = 0.1) -> RV:
@@ -770,7 +766,18 @@ class Connection(BaseConnection[Row]):
         The function must be used on generators that don't change connection
         fd (i.e. not on connect and reset).
         """
-        return waiting.wait(gen, self.pgconn.socket, timeout=timeout)
+        try:
+            return waiting.wait(gen, self.pgconn.socket, timeout=timeout)
+        except KeyboardInterrupt:
+            # On Ctrl-C, try to cancel the query in the server, otherwise
+            # otherwise the connection will be stuck in ACTIVE state
+            c = self.pgconn.get_cancel()
+            c.cancel()
+            try:
+                waiting.wait(gen, self.pgconn.socket, timeout=timeout)
+            except e.QueryCanceled:
+                pass  # as expected
+            raise
 
     @classmethod
     def _wait_conn(cls, gen: PQGenConn[RV], timeout: Optional[int]) -> RV:
